%%% bar20.m :: ver2018.2.20
clear all;
%%%
%%% define NN : Number of nodes per element
%%%
NN=2;
%%%
%%% stiffness :: k_bar1d2
%%%
function stf=k_bar1d2(node,ee,aa)
	length=abs(node(2)-node(1));
	stf(1,1)= ee*aa/length;
	stf(1,2)=-ee*aa/length;
	stf(2,1)=-ee*aa/length;
	stf(2,2)= ee*aa/length;
end
%%%
%%% stress recovery:: stress_bar1d2
%%%
function sgm=stress_bar1d2(node,ee,disp)
	length=node(2)-node(1);
	sgm=ee/length*(-disp(1)+disp(2));
end
%%%
%%% body force :: bf_bar1d2
%%%
function bf=bf_bar1d2(node,aa,load)
	length=node(2)-node(1);
	bf(1)=load*aa*length*0.5;
	bf(2)=load*aa*length*0.5;
end
%%%
%%% Node definitions:
%%%
numnp=21;
for i=1:21
	node(i)=(i-1)/20.0;
end
printf("///node :: numnp=%d\n",numnp);
for i=1:numnp
	printf("%d %e\n",i,node(i));
end
%%%
%%% Element definitions:
%%%
nume=20;
for i=1:20
	element(i,:)=[ i   i+1];
end
printf("///element :: nume=%d\n",nume);
for i=1:nume
	for j=1:NN
		printf("%d ",element(i,j) );
	end
	printf("\n");
end
%%%
%%% Material definitions:
%%%
E=1.0;
A=1.0;
printf("///material :: E=%e, A=%e\n",E,A);
%%%
%%% SPC definitions:
%%%
nspc=1;
spc(1)=numnp;
printf("///spc :: nspc=%d\n",nspc);
for i=1:nspc
	printf("%d %d\n",i,spc(i));
end
%%%
%%% load definitions:
%%%
ncload=0;
cload(1)=1;vcload(1)=1.0;
printf("///cload :: ncload=%d\n",ncload);
for i=1:ncload
	printf("%d %d %e\n",i,cload(i),vcload(i));
end
ndload=nume;
for i=1:ndload
	dload(i)=i;vdload(i)=-1.0;
end
printf("///dload :: ndload=%d\n",ndload);
for i=1:ndload
	printf("%d %d %e\n",i,dload(i),vdload(i));
end
%%%
%%% Global stiffness numnp x numnp
%%%
KG=zeros(numnp,numnp);
%%%
%%% Loop over elements
%%%
for ie=1:nume
%%% Local element coordinate
	for i=1:NN
		nodeL(i)=node(element(ie,i));
	end
%%% Local elememt stiffness element
	kL=k_bar1d2(nodeL,E,A);
%%% Add element 1 to Global stiffness 
	for i=1:NN
		irow=element(ie,i);
		for j=1:NN
			icol=element(ie,j);
			KG(irow,icol)=KG(irow,icol)+kL(i,j);
		end
	end
end
%%%
%%% Load vector (concentrated load)
%%%
FC=zeros(numnp,1);
for i=1:ncload
	npos=cload(i);
	FC(npos)=FC(npos)+vcload(i);
end
%%%
%%% Load vector (distributed load)
%%%
FD=zeros(numnp,1);
for i=1:ndload
	ie=dload(i);
	for j=1:NN
		nodeL(j)=node(element(ie,j));
	end
	bf=bf_bar1d2(nodeL,A,vdload(i));
	for j=1:NN
		irow=element(ie,j);
		FD(irow)=FD(irow)+bf(j);
	end
end
%%%
%%% TOTAL LOAD
%%%
	F=FC+FD;
%%%
%%% constrained conditions
%%%
count=1;
for i=1:nspc
	npos=spc(i);
	row(count,:)=KG(npos,:);
	KG(npos,:)=0;KG(:,npos)=0;KG(npos,npos)=1;
	F(npos)=0.0;
	count=count+1;
end
%%%
%%% computation 
%%%
printf("### Computation started\n");
U=KG\F;
printf("### Computation finished\n");
printf("### Nodal Displacement\n");
for i=1:numnp
	printf("%d %e\n",i,U(i));
end
%%% post processing :: stress
printf("### Element stress\n");
for ie=1:nume
	for j=1:NN
		nodeL(j)=node(element(ie,j));
		dispL(j)=U(element(ie,j));
	end
	printf("%d %e\n",ie,stress_bar1d2(nodeL,E,dispL));
end
%%% post processing :: reaction force
printf("### Reaction force\n");
count=1;
for i=1:nspc
	f=-FD(spc(i));
	for k=1:numnp
		f=f+row(count,k)*U(k);
	end
	count=count+1;
	printf("%d %e\n",spc(i),f);
end
%%%
%%% finished
%%%
