%%% beam20P.m :: ver2018.2.20
clear all;
%%%
%%% NDIM definitions: Dimension of problem
%%%
NDIM=1;
%%%
%%% NDOF definitions: Number of DOFs per element
%%%
NDOF=2;
%%%
%%% NN definitions: Number of nodes per element
%%%
NN=2;
%%%
%%% stiffeness :: k_beam1d2
%%%
function stf=k_beam1d2(node,ee,ii)
	length=abs(node(2)-node(1));
	k=ee*ii/length^3;
	stf(1,1)= k*12;
	stf(1,2)= k*6*length;
	stf(1,3)=-k*12;
	stf(1,4)= k*6*length;
	stf(2,1)= k*6*length;
	stf(2,2)= k*4*length^2;
	stf(2,3)=-k*6*length;
	stf(2,4)= k*2*length^2;
	stf(3,1)=-k*12;
	stf(3,2)=-k*6*length;
	stf(3,3)= k*12;
	stf(3,4)=-k*6*length;
	stf(4,1)= k*6*length;
	stf(4,2)= k*2*length^2;
	stf(4,3)=-k*6*length;
	stf(4,4)= k*4*length^2;
end
%%%
%%% internal force recovery :: mq_beam1d2
%%%
function [m q]=mq_beam1d2(node,ee,ii,disp)
	length=abs(node(2)-node(1));
%%% displacement
	d(1)=disp(1,1);
	d(2)=disp(1,2);
	d(3)=disp(2,1);
	d(4)=disp(2,2);
%%% node-1
	n(1)=-6/length^2;
	n(2)=-4/length;
	n(3)= 6/length^2;
	n(4)=-2/length;
	m(1)=0.0;
	for i=1:4
		m(1)=m(1)+n(i)*d(i);
	end
	m(1)=-ee*ii*m(1);
%%% node-2
	n(1)= 6/length^2;
	n(2)= 2/length;
	n(3)=-6/length^2;
	n(4)= 4/length;
	m(2)=0.0;
	for i=1:4
		m(2)=m(2)+n(i)*d(i);
	end
	m(2)=-ee*ii*m(2);
%%% shear force
	n(1)= 12/length^3;
	n(2)=  6/length^2;
	n(3)=-12/length^3;
	n(4)=  6/length^2;
	q=0.0;
	for i=1:4
		q=q+n(i)*d(i);
	end
	q=-ee*ii*q;
end
%%%
%%% body force :: bf_beam1d2
%%%
function bf=bf_beam1d2(node,p)
	length=abs(node(2)-node(1));
	bf(1)=-p*length/2;
	bf(2)=-p*length^2/12;
	bf(3)=-p*length/2;
	bf(4)= p*length^2/12;
end
%%%
%%% VTK :: writeVTK
%%%
function writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U)
%%% VTK_CELL_TYPE 
	vtk_cell_type=3;
%%% NVARS
	NVARS=0; 
	vname=char("S11");
%%% Header 
	fprintf(fid,"# vtk DataFile Version 2.0\n");
	fprintf(fid,"%s\n",program_name());
	fprintf(fid,"ASCII\n");
%%% Node 
	fprintf(fid,"DATASET UNSTRUCTURED_GRID\n");
	fprintf(fid,"POINTS %d float\n",numnp);
	for i=1:numnp
		xc(1)=0.0;xc(2)=0.0;xc(3)=0.0;
		for j=1:NDIM
			xc(j)=node(i,j);
		end
		fprintf(fid,"%e %e %e\n",xc(1),xc(2),xc(3));
	endfor
%%% Element
	fprintf(fid,"CELLS %d %d\n",nume,nume*(1+NN));
	for i=1:nume
		for j=1:NN
			nid(j)=element(i,j);
		end
		fprintf(fid,"%d ",NN);
		for j=1:NN
			fprintf(fid,"%d ",nid(j)-1);
		end
		fprintf(fid,"\n");
	end
%%% type
	fprintf(fid,"CELL_TYPES %d\n",nume);
	for i=1:nume
		fprintf(fid,"%d\n",vtk_cell_type);
	end
%%% Result for NODE
	fprintf(fid,"POINT_DATA %d\n",numnp);
%%% Deformation 
	fprintf(fid,"VECTORS DISPLACEMENT float\n");
	for i=1:numnp
		if( NDIM == 1)
			fprintf(fid," 0.0 0.0 %e\n",U(2*i-1));
		endif
	end
end
%%%
%%% Node definitions: 
%%%
numnp=21;
for i=1:21
        node(i,:)=[(i-1)/20.0];
end
printf("///node :: numnp=%d\n",numnp);
for i=1:numnp
	printf("%d ",i);
	for j=1:NDIM
		printf("%e ",node(i,j));
	end
	printf("\n");
end
%%%
%%% Element definitions:
%%%
nume=20;
for i=1:nume
        element(i,:)=[ i   i+1];
end
printf("///element :: nume=%d\n",nume);
for i=1:nume
	printf("%d ",i);
	for j=1:NN
		printf("%d ",element(i,j) );
	end
	printf("\n");
end
%%%
%%% Material definitions:
%%%
E=1.0;
I=1.0;
printf("///material :: E=%e, I=%e\n",E,I);
%%%
%%% SPC definitions:
%%%
nspc=4;
spc(1,:)=[1 1];
spc(2,:)=[1 2];
spc(3,:)=[numnp 1];
spc(4,:)=[numnp 2];
printf("///spc :: nspc=%d\n",nspc);
for i=1:nspc
	printf("%d %d %d\n",i,spc(i,1),spc(i,2))
end
%%%
%%% load definitions:
%%%
ncload=0;
cload(1,:)=[2 1];vcload(1)=-1.0;
printf("///cload :: ncload=%d\n",ncload);
for i=1:ncload
	printf("%d %d %d %e\n",i,cload(i,1),cload(i,2),vcload(i))
end
ndload=nume;
for i=1:ndload
	dload(i)=i;
	vdload(i)=1.0;
end
printf("///dload :: ndload=%d\n",ndload);
for i=1:ndload
	printf("%d %d %e\n",i,dload(i),vdload(i))
end
%%%
%%% Global stiffness (NDOF*numnp) x (NDOF*numnp)
%%%
KG=zeros(NDOF*numnp,NDOF*numnp);
%%%
%%% Loop over elements
%%%
for ie=1:nume
%%% Local element coordinate
	for i=1:NN
		for j=1:NDIM
			nodeL(i,j)=node(element(ie,i),j);
		end
	end
%%% Local elememt stiffness element
	kL=k_beam1d2(nodeL,E,I);
%%% Add element 1 to Global stiffness 
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			for j=1:NN
				for jj=1:NDOF
					icol=NDOF*(element(ie,j)-1)+jj;
					KG(irow,icol)=KG(irow,icol)+kL(NDOF*(i-1)+ii,NDOF*(j-1)+jj);
				end
			end
		end
	end
end
%%%
%%% Load vector (condentrated load)
%%%
FC=zeros(NDOF*numnp,1);
for i=1:ncload
	npos=NDOF*(cload(i,1)-1)+cload(i,2)
	FC(npos)=FC(npos)+vcload(i);
end
%%%
%%% Load vector (distributed load)
%%%
FD=zeros(NDOF*numnp,1);
for i=1:ndload
	ie=dload(i);
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
	end
	bf=bf_beam1d2(nodeL,vdload(i));
	for j=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,j)-1)+ii;
			FD(irow)=FD(irow)+bf(NDOF*(j-1)+ii);
		end
	end
end
%%%
%%% TOTAL LOAD
%%%
	F=FC+FD;
%%%
%%% constrained conditions
%%%
count=1;
for i=1:nspc
	npos=NDOF*(spc(i,1)-1)+spc(i,2);
	row(count,:)=KG(npos,:);
	KG(npos,:)=0;
	KG(:,npos)=0;
	KG(npos,npos)=1;
	F(npos)=0.0;
	count=count+1;
end
%%%
%%% computation 
%%%
printf("### Computation started\n");
U=KG\F;
printf("### Computation finished\n");
printf("//Result displacement\n")
for i=1:numnp
	printf("%d ",i);
	for j=1:NDOF
		printf("%e ",U(NDOF*(i-1)+j));
	end
	printf("\n");
end
%%% post processing :: stress
printf("Element stress\n")
for ie=1:nume
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
		for k=1:NDOF
			dispL(j,k)=U(NDOF*(element(ie,j)-1)+k);
		end
	end
	[ m q ]=mq_beam1d2(nodeL,E,I,dispL);
	printf("%d M: %e %e Q: %e \n",ie,m(1),m(2),q );
end
%%% post processing :: reaction force
printf("Reaction force\n")
count=1;
for i=1:nspc
	f=-FD(NDOF*(spc(i,1)-1)+spc(i,2));
	for k=1:NDOF*numnp
		f=f+row(count,k)*U(k);
	end
	count=count+1;
	printf("%d %d %e\n",spc(i,1),spc(i,2),f);
end
%%%
%%% for visualization
%%%
fid=fopen('vtk.vtk','w');
writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U);
fclose(fid);
%%%
%%% finished
%%%
