%%% triaFEM.m :: ver2019.9.2
clear all;
%%%
%%% NDIM definitions: Dimension of problem
%%%
NDIM=2;
%%%
%%% NDOF definitions: Number of DOFs per element
%%%
NDOF=2;
%%%
%%% NN definitions: Number of nodes per element
%%%
NN=3;
%%%
%%% stiffeness :: k_tria2d3
%%%
function stf=k_tria2d3(node,type,ee,poi)
%%% set [D]
	if( type == 0 )
		coef1=ee/((1.0+poi)*(1.0-2.0*poi));
		coef2=ee/(2.0*(1.0+poi));
		D(1,1)=coef1*(1.0-poi);
		D(1,2)=coef1*poi;
		D(1,3)=0.0;
		D(2,1)=coef1*poi;
		D(2,2)=coef1*(1.0-poi);
		D(2,3)=0.0;
		D(3,1)=0.0;
		D(3,2)=0.0;
		D(3,3)=coef2;
	else
		coef1=ee/(1-poi*poi);
		coef2=0.5*(1-poi);
		D(1,1)=coef1;
		D(1,2)=coef1*poi;
		D(1,3)=0.0;
		D(2,1)=D(1,2);
		D(2,2)=D(1,1);
		D(2,3)=0.0;
		D(3,1)=0.0;
		D(3,2)=0.0;
		D(3,3)=coef1*coef2;
	end
%%% constants
	a(1)=node(2,1)*node(3,2)-node(3,1)*node(2,2);
	a(2)=node(3,1)*node(1,2)-node(1,1)*node(3,2);
	a(3)=node(1,1)*node(2,2)-node(2,1)*node(1,2);
	b(1)=node(2,2)-node(3,2);
	b(2)=node(3,2)-node(1,2);
	b(3)=node(1,2)-node(2,2);
	c(1)=node(3,1)-node(2,1);
	c(2)=node(1,1)-node(3,1);
	c(3)=node(2,1)-node(1,1);
	area=(a(1)+a(2)+a(3))*0.5;
	const1=0.5/area;
%%% shape function
	for i=1:3
		hx(i)=b(i)*const1;
		hy(i)=c(i)*const1;
	end
%%% set [B]
	B=zeros(3,6);
	for i=1:3
		B(1,2*i-1)=hx(i);
		B(2,2*i  )=hy(i);
		B(3,2*i-1)=hy(i);
		B(3,2*i  )=hx(i);
	end
%%% Calculate Stiffness
	stf=B'*D*B*area;
end
%%%
%%%  stress recovery :: rcv_tria2d3
%%%
function [s e]=rcv_tria2d3(node,type,ee,poi,disp)
%%% set [D]
	if( type == 0 )
		coef1=ee/((1.0+poi)*(1.0-2.0*poi));
		coef2=ee/(2.0*(1.0+poi));
		D(1,1)=coef1*(1.0-poi);
		D(1,2)=coef1*poi;
		D(1,3)=0.0;
		D(2,1)=coef1*poi;
		D(2,2)=coef1*(1.0-poi);
		D(2,3)=0.0;
		D(3,1)=0.0;
		D(3,2)=0.0;
		D(3,3)=coef2;
	else
		coef1=ee/(1-poi*poi);
		coef2=0.5*(1-poi);
		D(1,1)=coef1;
		D(1,2)=coef1*poi;
		D(1,3)=0.0;
		D(2,1)=D(1,2);
		D(2,2)=D(1,1);
		D(2,3)=0.0;
		D(3,1)=0.0;
		D(3,2)=0.0;
		D(3,3)=coef1*coef2;
	end
%%% constants
	a(1)=node(2,1)*node(3,2)-node(3,1)*node(2,2);
	a(2)=node(3,1)*node(1,2)-node(1,1)*node(3,2);
	a(3)=node(1,1)*node(2,2)-node(2,1)*node(1,2);
	b(1)=node(2,2)-node(3,2);
	b(2)=node(3,2)-node(1,2);
	b(3)=node(1,2)-node(2,2);
	c(1)=node(3,1)-node(2,1);
	c(2)=node(1,1)-node(3,1);
	c(3)=node(2,1)-node(1,1);
	area=(a(1)+a(2)+a(3))*0.5;
	const1=0.5/area;
%%% shape function
	for i=1:3
		hx(i)=b(i)*const1;
		hy(i)=c(i)*const1;
	end
%%% set [B]
	B=zeros(3,6);
	for i=1:3
		B(1,2*i-1)=hx(i);
		B(2,2*i  )=hy(i);
		B(3,2*i-1)=hy(i);
		B(3,2*i  )=hx(i);
	end
%%% Local displacement
	dd(1)=disp(1,1);
	dd(2)=disp(1,2);
	dd(3)=disp(2,1);
	dd(4)=disp(2,2);
	dd(5)=disp(3,1);
	dd(6)=disp(3,2);
%%% stress
	e=B*dd';
	s=D*e;
	s(4)=0.0;
	if( type == 0 )
		s(4)=poi*(s(1)+s(2));
	end
end
%%%
%%% distributed load :: dload_tria2d3
%%%
function vect=dload_tria2d3(node,ltype,val)
%%% ltype=1  BX: BODY FORCE IN X-DIRECTION
%%% ltype=2  BY: BODY FORCE IN Y-DIRECTION
%%% ltype=10 P1: TRACTION IN NORMAL-DIRECTION FOR FACE-1
%%% ltype=20 P2: TRACTION IN NORMAL-DIRECTION FOR FACE-2
%%% ltype=30 P3: TRACTION IN NORMAL-DIRECTION FOR FACE-3
%%% LOAD TYPE
	ivol=0;
	isuf=0;
	if( ltype < 10 )
		ivol=1;
	else
		isuf=1;
	end
%%% CLEAR
	vect=zeros(3*2,1);
%%% SURFACE LOAD
	if( isuf == 1 )
		switch( ltype )
			case (10)
				nod1=1;nod2=2;
			case (20)
				nod1=2;nod2=3;
			case (30)
				nod1=3;nod2=1;
		end
		px=val*(node(nod1,2)-node(nod2,2));
		py=val*(node(nod2,1)-node(nod1,1));
		vect(2*nod1-1)=px*0.5;
		vect(2*nod1  )=py*0.5;
		vect(2*nod2-1)=px*0.5;
		vect(2*nod2  )=py*0.5;
	end
%%% VOLUME LOAD
	if( ivol == 1 )
		a1=node(2,1)*node(3,2)-node(3,1)*node(2,2);
		a2=node(3,1)*node(1,2)-node(1,1)*node(3,2);
		a3=node(1,1)*node(2,2)-node(2,1)*node(1,2);
		area=(a1+a2+a3)*0.5;
		switch( ltype )
			case (1)
			vect(1)=val*area/3.0;
			vect(2)=0.0;
			vect(3)=val*area/3.0;
			vect(4)=0.0;
			vect(5)=val*area/3.0;
			vect(6)=0.0;
			case (2)
			vect(1)=0.0;
			vect(2)=val*area/3.0;
			vect(3)=0.0;
			vect(4)=val*area/3.0;
			vect(5)=0.0;
			vect(6)=val*area/3.0;
		end
	end
end
%%%
%%% VTK :: writeVTK
%%%
function writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,n_stress)
%%% VTK_CELL_TYPE 
	vtk_cell_type=5;
%%% NVARS
	NVARS=4; 
	vname=char("S11","S22","S12","S33");
%%% Header 
	fprintf(fid,"# vtk DataFile Version 2.0\n");
	fprintf(fid,"%s\n",program_name());
	fprintf(fid,"ASCII\n");
%%% Node 
	fprintf(fid,"DATASET UNSTRUCTURED_GRID\n");
	fprintf(fid,"POINTS %d float\n",numnp);
	for i=1:numnp
		xc(1)=0.0;xc(2)=0.0;xc(3)=0.0;
		for j=1:NDIM
			xc(j)=node(i,j);
		end
		fprintf(fid,"%e %e %e\n",xc(1),xc(2),xc(3));
	end
%%% Element
	fprintf(fid,"CELLS %d %d\n",nume,nume*(1+NN));
	for i=1:nume
		for j=1:NN
			nid(j)=element(i,j);
		end
		fprintf(fid,"%d ",NN);
		for j=1:NN
			fprintf(fid,"%d ",nid(j)-1);
		end
		fprintf(fid,"\n");
	end
%%% type
	fprintf(fid,"CELL_TYPES %d\n",nume);
	for i=1:nume
		fprintf(fid,"%d\n",vtk_cell_type);
	end
%%% Result 
	fprintf(fid,"POINT_DATA %d\n",numnp);
%%% Deformation 
	fprintf(fid,"VECTORS DISPLACEMENT float\n");
	for i=1:numnp
		if( NDIM == 2)
			fprintf(fid," %e %e 0.0\n",U(2*i-1),U(2*i));
		end
	end
%%% Nodal Stress 
	for i=1:NVARS
		fprintf(fid,"SCALARS %s float 1\n",vname(i,:));
		fprintf(fid,"LOOKUP_TABLE default\n");
		for j=1:numnp
			fprintf(fid,"%e\n",n_stress(j,i));
		end
	end
end
%%%
%%% Read FEM model data
%%%
function [numnp,node,nume,element,type,e,nu,nspc,spc,ncload,cload,vcload,ndload,dload,vdload]\
=readFEM(fid,NDIM,NN,NDOF)
%%%
%%% Node definitions:
%%%
numnp=fscanf(fid,"%d",'C');
nume=fscanf(fid,"%d",'C');
nspc=fscanf(fid,"%d",'C');
ncload=fscanf(fid,"%d",'C');
ndload=fscanf(fid,"%d",'C');
for i=1:numnp
	for j=1:NDIM
		node(i,j)=fscanf(fid,"%lf",'C');
	end
end
printf("///node :: numnp=%d\n",numnp);
for i=1:numnp
	printf("%d ",i);
	for j=1:NDIM
		printf("%e ",node(i,j));
	end
	printf("\n");
end
%%%
%%% Element definitions:
%%%
for i=1:nume
	dummy=fscanf(fid,"%d",'C');
	for j=1:NN
		element(i,j)=fscanf(fid,"%d",'C');
	end
end
printf("///element :: nume=%d\n",nume);
for i=1:nume
	printf("%d ",i);
	for j=1:NN
		printf("%d ",element(i,j) );
	end
	printf("\n");
end
%%%
%%% Material definitions:
%%%
type=fscanf(fid,"%d",'C');
e=fscanf(fid,"%lf",'C');
nu=fscanf(fid,"%lf",'C');
printf("///material :: type=%d E=%e NU=%e\n",type,e,nu);
%%%
%%% SPC definitions:
%%%
for i=1:nspc
	spc(i,1)=fscanf(fid,"%d",'C');
	spc(i,2)=fscanf(fid,"%d",'C');
end
printf("///spc :: nspc=%d\n",nspc);
for i=1:nspc
	printf("%d %d %d\n",i,spc(i,1),spc(i,2))
end
%%%
%%% load definitions:
%%%
cload(1,1)=0
cload(1,2)=0
vcload(1)=0.0
for i=1:ncload
	cload(i,1)=fscanf(fid,"%d",'C');
	cload(i,2)=fscanf(fid,"%d",'C');
	vcload(i) =fscanf(fid,"%lf",'C');
end
printf("///cload :: ncload=%d\n",ncload);
for i=1:ncload
	printf("%d %d %d %e\n",i,cload(i,1),cload(i,2),vcload(i))
end
dload(1,1)=0;
dload(1,2)=0;
vdload(1)=0.0;
for i=1:ndload
	dload(i,1)=fscanf(fid,"%d",'C');
	dload(i,2)=fscanf(fid,"%d",'C');
	vdload(i)=fscanf(fid,"%lf",'C');
end
printf("///dload :: ndload=%d\n",ndload);
for i=1:ndload
	printf("%d %d %d %e\n",i,dload(i,1),dload(i,2),vdload(i))
end
end
%%%
%%%  Read FEM model data
%%%
finp=fopen('pwhCPS3.dat','r');
[numnp,node,nume,element,TYPE,E,NU,nspc,spc,ncload,cload,vcload,ndload,dload,vdload]\
=readFEM(finp,NDIM,NN,NDOF);
%%%
%%% Global stiffness (NDOF*numnp) x (NDOF*numnp)
%%%
KG=zeros(NDOF*numnp,NDOF*numnp);
%%%
%%% Loop over elements
%%%
for ie=1:nume
%%% Local element coordinate
	for i=1:NN
		for j=1:NDIM
			nodeL(i,j)=node(element(ie,i),j);
		end
	end
%%% Local elememt stiffness element
	kL=k_tria2d3(nodeL,TYPE,E,NU);
%%% Add element 1 to Global stiffness 
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			for j=1:NN
				for jj=1:NDOF
					icol=NDOF*(element(ie,j)-1)+jj;
					KG(irow,icol)=KG(irow,icol)+kL(NDOF*(i-1)+ii,NDOF*(j-1)+jj);
				end
			end
		end
	end
end
%%%
%%% Load vector (condentrated load)
%%%
FC=zeros(NDOF*numnp,1);
for i=1:ncload
	npos=NDOF*(cload(i,1)-1)+cload(i,2);
	FC(npos)=FC(npos)+vcload(i);
end
%%%
%%% Load vector (distributed load)
%%%
FD=zeros(NDOF*numnp,1);
for i=1:ndload
	ie=dload(i,1);
	ltype=dload(i,2);
	val=vdload(i);
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
	end
	df=dload_tria2d3(nodeL,ltype,val);
	for j=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,j)-1)+ii;
			FD(irow)=FD(irow)+df(NDOF*(j-1)+ii);
		end
	end
end
%%%
%%% TOTAL LOAD
%%%
	F=FC+FD;
%%%
%%% constrained conditions
%%%
count=1;
for i=1:nspc
	npos=NDOF*(spc(i,1)-1)+spc(i,2);
	row(count,:)=KG(npos,:);
	KG(npos,:)=0;
	KG(:,npos)=0;
	KG(npos,npos)=1;
	F(npos)=0.0;
	count=count+1;
end
%%%
%%% computation 
%%%
printf("### Computation started\n");
U=KG\F;
printf("### Computation finished\n");
printf("//Result displacement\n")
for i=1:numnp
	printf("%d ",i);
	for j=1:NDOF
		printf("%e ",U(NDOF*(i-1)+j));
	end
	printf("\n");
end
%%% post processing :: stress and strain
%%% for nodal stress/S11,S22,S12,S33
n_stress=zeros(numnp,4);
n_count=zeros(numnp);
printf("Element stress/strain\n")
for ie=1:nume
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
		for k=1:NDOF
			dispL(j,k)=U(NDOF*(element(ie,j)-1)+k);
		end
	end
	[s e]=rcv_tria2d3(nodeL,TYPE,E,NU,dispL);
	printf("%d S11/S22/S12/S33: %e %e %e %e\n",ie,s(1),s(2),s(3),s(4));
	printf("%d E11/E22/E12: %e %e %e\n",ie,e(1),e(2),e(3));
%%% add S11,S22,S12,S33
	for j=1:NN
		n_stress(element(ie,j),1)=n_stress(element(ie,j),1)+s(1);
		n_stress(element(ie,j),2)=n_stress(element(ie,j),2)+s(2);
		n_stress(element(ie,j),3)=n_stress(element(ie,j),3)+s(3);
		n_stress(element(ie,j),4)=n_stress(element(ie,j),4)+s(4);
		n_count(element(ie,j))=n_count(element(ie,j))+1;
	end
end
printf("Nodal stress/S11 S22 S12 S33\n")
for i=1:numnp
	n_stress(i,1)=n_stress(i,1)/n_count(i);
	n_stress(i,2)=n_stress(i,2)/n_count(i);
	n_stress(i,3)=n_stress(i,3)/n_count(i);
	n_stress(i,4)=n_stress(i,4)/n_count(i);
	printf("%d %e %e %e %e\n",i,n_stress(i,1),n_stress(i,2),n_stress(i,3),n_stress(i,4));
end
%%% post processing :: reaction force
printf("Reaction force\n")
count=1;
for i=1:nspc
	f=-FD(NDOF*(spc(i,1)-1)+spc(i,2));
	for k=1:NDOF*numnp
		f=f+row(count,k)*U(k);
	end
	count=count+1;
	printf("%d %d %e\n",spc(i,1),spc(i,2),f);
end
%%%
%%% for visualization
%%%
	fid=fopen('vtk.vtk','w');
	writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,n_stress);
	fclose(fid);
%%%
%%% finished
%%%
