%%% hexaFEM.m :: ver2019.9.18
clear all;
%%%
%%% NDIM definitions: Dimension of problem
%%%
NDIM=3;
%%%
%%% NDOF definitions: Number of DOFs per element
%%%
NDOF=3;
%%%
%%% NN definitions: Number of nodes per element
%%%
NN=8;
%%%
%%% stiffeness :: k_hexa3d8
%%%
function stf=k_hexa3d8(node,ee,poi)
%%% set [D]
D=zeros(6,6);
D(1,1)=ee*(1-poi)/(1-2*poi)/(1+poi);
D(1,2)=ee*poi/(1-2*poi)/(1+poi);
D(1,3)=D(1,2);
D(2,1)=D(1,2);
D(2,2)=D(1,1);
D(2,3)=D(1,2);
D(3,1)=D(1,3);
D(3,2)=D(2,3);
D(3,3)=D(1,1);
D(4,4)=ee/(1+poi)*0.5;
D(5,5)=ee/(1+poi)*0.5;
D(6,6)=ee/(1+poi)*0.5;
%%% allocate
stf=zeros(24,24);
B=zeros(6,24);
%%% integration
xg=[-1/sqrt(3), 1/sqrt(3)];
wg=[1,1];
for i=1:2
	r1=xg(i);
	for j=1:2
		r2=xg(j);
		for k=1:2
			r3=xg(k);
%%% interpolation function
			r1p=1+r1;
		  	r2p=1+r2;
		  	r3p=1+r3;
		  	r1m=1-r1;
		  	r2m=1-r2;
		  	r3m=1-r3;
		  	N(1)=r1m*r2m*r3m/8;
		  	N(2)=r1p*r2m*r3m/8;
		  	N(3)=r1p*r2p*r3m/8;
		  	N(4)=r1m*r2p*r3m/8;
		  	N(5)=r1m*r2m*r3p/8;
		  	N(6)=r1p*r2m*r3p/8;
		  	N(7)=r1p*r2p*r3p/8;
		  	N(8)=r1m*r2p*r3p/8;
%%% derivative of interpolation function  for r1
		  	Nr1(1)=-r2m*r3m/8;
		  	Nr1(2)= r2m*r3m/8;
		  	Nr1(3)= r2p*r3m/8;
		  	Nr1(4)=-r2p*r3m/8;
		  	Nr1(5)=-r2m*r3p/8;
		  	Nr1(6)= r2m*r3p/8;
		  	Nr1(7)= r2p*r3p/8;
		  	Nr1(8)=-r2p*r3p/8;
%%% derivative of interpolation function  for r2
		  	Nr2(1)=-r1m*r3m/8;
		  	Nr2(2)=-r1p*r3m/8;
		  	Nr2(3)= r1p*r3m/8;
		  	Nr2(4)= r1m*r3m/8;
		  	Nr2(5)=-r1m*r3p/8;
		  	Nr2(6)=-r1p*r3p/8;
		  	Nr2(7)= r1p*r3p/8;
		  	Nr2(8)= r1m*r3p/8;
%%% derivative of interpolation function  for r3
		  	Nr3(1)=-r1m*r2m/8;
		  	Nr3(2)=-r1p*r2m/8;
		  	Nr3(3)=-r1p*r2p/8;
		  	Nr3(4)=-r1m*r2p/8;
		  	Nr3(5)= r1m*r2m/8;
		  	Nr3(6)= r1p*r2m/8;
		  	Nr3(7)= r1p*r2p/8;
		  	Nr3(8)= r1m*r2p/8;
%%% Jacobi matrix 
			J=zeros(3,3);
			for l=1:8
				J(1,1)=J(1,1)+Nr1(l)*node(l,1);
				J(1,2)=J(1,2)+Nr1(l)*node(l,2);
				J(1,3)=J(1,3)+Nr1(l)*node(l,3);
				J(2,1)=J(2,1)+Nr2(l)*node(l,1);
				J(2,2)=J(2,2)+Nr2(l)*node(l,2);
				J(2,3)=J(2,3)+Nr2(l)*node(l,3);
				J(3,1)=J(3,1)+Nr3(l)*node(l,1);
				J(3,2)=J(3,2)+Nr3(l)*node(l,2);
				J(3,3)=J(3,3)+Nr3(l)*node(l,3);
			end
%%% inverse of Jacobian
			Ji=inv(J);
%%% B matrix
			for l=1:8
				dndx=Ji(1,1)*Nr1(l)+Ji(1,2)*Nr2(l)+Ji(1,3)*Nr3(l);
				dndy=Ji(2,1)*Nr1(l)+Ji(2,2)*Nr2(l)+Ji(2,3)*Nr3(l);
				dndz=Ji(3,1)*Nr1(l)+Ji(3,2)*Nr2(l)+Ji(3,3)*Nr3(l);
				B(1,3*l-2)=dndx;
				B(2,3*l-2)=0.0;
				B(3,3*l-2)=0.0;
				B(4,3*l-2)=dndy;
				B(5,3*l-2)=0.0;
				B(6,3*l-2)=dndz;
				B(1,3*l-1)=0.0;
				B(2,3*l-1)=dndy;
				B(3,3*l-1)=0.0;
				B(4,3*l-1)=dndx;
				B(5,3*l-1)=dndz;
				B(6,3*l-1)=0.0;
				B(1,3*l  )=0.0;
				B(2,3*l  )=0.0;
				B(3,3*l  )=dndz;
				B(4,3*l  )=0.0;
				B(5,3*l  )=dndy;
				B(6,3*l  )=dndx;
			end
%%% stiffness
			stf=stf+B'*D*B*det(J)*wg(i)*wg(j)*wg(k);
		end
	end
end
end
%%%
%%% stiffeness :: rcv_hexa3d8
%%%
function [s e]=rcv_hexa3d8(node,ee,poi,disp)
%%% set [D]
	D=zeros(6,6);
	D(1,1)=ee*(1-poi)/(1-2*poi)/(1+poi);
	D(1,2)=ee*poi/(1-2*poi)/(1+poi);
	D(1,3)=D(1,2);
	D(2,1)=D(1,2);
	D(2,2)=D(1,1);
	D(2,3)=D(1,2);
	D(3,1)=D(1,3);
	D(3,2)=D(2,3);
	D(3,3)=D(1,1);
	D(4,4)=ee/(1+poi)*0.5;
	D(5,5)=ee/(1+poi)*0.5;
	D(6,6)=ee/(1+poi)*0.5;
%%% set disp
	for i=1:8
		dd(3*i-2)=disp(i,1);
		dd(3*i-1)=disp(i,2);
		dd(3*i  )=disp(i,3);
	end
	B=zeros(6,24);
%%% integration
	xg=[-1/sqrt(3), 1/sqrt(3)];
	count=1;
	for i=1:2
		r1=xg(i);
		for j=1:2
			r2=xg(j);
			for k=1:2
				r3=xg(k);
%%% interpolation function
				r1p=1+r1;
		  		r2p=1+r2;
		  		r3p=1+r3;
		  		r1m=1-r1;
		  		r2m=1-r2;
		  		r3m=1-r3;
		  		N(1)=r1m*r2m*r3m/8;
		  		N(2)=r1p*r2m*r3m/8;
		  		N(3)=r1p*r2p*r3m/8;
		  		N(4)=r1m*r2p*r3m/8;
		  		N(5)=r1m*r2m*r3p/8;
		  		N(6)=r1p*r2m*r3p/8;
		  		N(7)=r1p*r2p*r3p/8;
		  		N(8)=r1m*r2p*r3p/8;
%%% derivative of interpolation function  for r1
		  		Nr1(1)=-r2m*r3m/8;
		  		Nr1(2)= r2m*r3m/8;
		  		Nr1(3)= r2p*r3m/8;
		  		Nr1(4)=-r2p*r3m/8;
		  		Nr1(5)=-r2m*r3p/8;
		  		Nr1(6)= r2m*r3p/8;
		  		Nr1(7)= r2p*r3p/8;
		  		Nr1(8)=-r2p*r3p/8;
%%% derivative of interpolation function  for r2
		  		Nr2(1)=-r1m*r3m/8;
		  		Nr2(2)=-r1p*r3m/8;
		  		Nr2(3)= r1p*r3m/8;
		  		Nr2(4)= r1m*r3m/8;
		  		Nr2(5)=-r1m*r3p/8;
		  		Nr2(6)=-r1p*r3p/8;
		  		Nr2(7)= r1p*r3p/8;
		  		Nr2(8)= r1m*r3p/8;
%%% derivative of interpolation function  for r3
		  		Nr3(1)=-r1m*r2m/8;
		  		Nr3(2)=-r1p*r2m/8;
		  		Nr3(3)=-r1p*r2p/8;
		  		Nr3(4)=-r1m*r2p/8;
		  		Nr3(5)= r1m*r2m/8;
		  		Nr3(6)= r1p*r2m/8;
		  		Nr3(7)= r1p*r2p/8;
		  		Nr3(8)= r1m*r2p/8;
%%% Jacobi matrix 
				J=zeros(3,3);
				for l=1:8
					J(1,1)=J(1,1)+Nr1(l)*node(l,1);
					J(1,2)=J(1,2)+Nr1(l)*node(l,2);
					J(1,3)=J(1,3)+Nr1(l)*node(l,3);
					J(2,1)=J(2,1)+Nr2(l)*node(l,1);
					J(2,2)=J(2,2)+Nr2(l)*node(l,2);
					J(2,3)=J(2,3)+Nr2(l)*node(l,3);
					J(3,1)=J(3,1)+Nr3(l)*node(l,1);
					J(3,2)=J(3,2)+Nr3(l)*node(l,2);
					J(3,3)=J(3,3)+Nr3(l)*node(l,3);
				end
%%% inverse of Jacobian
				Ji=inv(J);
%%% B matrix
				for l=1:8
					dndx=Ji(1,1)*Nr1(l)+Ji(1,2)*Nr2(l)+Ji(1,3)*Nr3(l);
					dndy=Ji(2,1)*Nr1(l)+Ji(2,2)*Nr2(l)+Ji(2,3)*Nr3(l);
					dndz=Ji(3,1)*Nr1(l)+Ji(3,2)*Nr2(l)+Ji(3,3)*Nr3(l);
					B(1,3*l-2)=dndx;
					B(2,3*l-2)=0.0;
					B(3,3*l-2)=0.0;
					B(4,3*l-2)=dndy;
					B(5,3*l-2)=0.0;
					B(6,3*l-2)=dndz;
					B(1,3*l-1)=0.0;
					B(2,3*l-1)=dndy;
					B(3,3*l-1)=0.0;
					B(4,3*l-1)=dndx;
					B(5,3*l-1)=dndz;
					B(6,3*l-1)=0.0;
					B(1,3*l  )=0.0;
					B(2,3*l  )=0.0;
					B(3,3*l  )=dndz;
					B(4,3*l  )=0.0;
					B(5,3*l  )=dndy;
					B(6,3*l  )=dndx;
				end
%%% stress & strain
				strain=B*dd';
				stress=D*strain;
				for l=1:6
					s(count,l)=stress(l);
					e(count,l)=strain(l);
				end
				count=count+1;
			end
		end
	end
end
%%%
%%% distributed load :: dload_hexa3d8
%%%
function vect=dload_hexa3d8(node,ltype,val)
%%% ltype=1  BX: BODY FORCE IN X-DIRECTION
%%% ltype=2  BY: BODY FORCE IN Y-DIRECTION
%%% ltype=3  BZ: BODY FORCE IN Z-DIRECTION
%%% ltype=10 P1: TRACTION IN NORMAL-DIRECTION FOR FACE-1
%%% ltype=20 P2: TRACTION IN NORMAL-DIRECTION FOR FACE-2
%%% ltype=30 P3: TRACTION IN NORMAL-DIRECTION FOR FACE-3
%%% ltype=40 P4: TRACTION IN NORMAL-DIRECTION FOR FACE-4
%%% ltype=50 P5: TRACTION IN NORMAL-DIRECTION FOR FACE-5
%%% ltype=60 P6: TRACTION IN NORMAL-DIRECTION FOR FACE-6
	ivol=0;
	isuf=0;
	if( ltype < 10 )
		ivol=1;
	else
		isuf=1;
	end
%%% CLEAR
	vect=zeros(24,1);
%%% integration
	xg=[-1/sqrt(3), 1/sqrt(3)];
	wg=[1,1];
%%% SURFACE LOAD
	if( isuf == 1 )
		switch( ltype )
			case (10)
			nL(1)=1;nL(2)=2;nL(3)=3;nL(4)=4;
			case (20)
			nL(1)=8;nL(2)=7;nL(3)=6;nL(4)=5;
			case (30)
			nL(1)=5;nL(2)=6;nL(3)=2;nL(4)=1;
			case (40)
			nL(1)=6;nL(2)=7;nL(3)=3;nL(4)=2;
			case (50)
			nL(1)=7;nL(2)=8;nL(3)=4;nL(4)=3;
			case (60)
			nL(1)=8;nL(2)=5;nL(3)=1;nL(4)=4;
		end
%%% 
		for i=1:2
			r1=xg(i);
			for j=1:2
				r2=xg(j);
%%% interpolation function
				r1p=1+r1;
				r2p=1+r2;
				r1m=1-r1;
				r2m=1-r2;
				N(1)=r1m*r2m/4;
				N(2)=r1p*r2m/4;
				N(3)=r1p*r2p/4;
				N(4)=r1m*r2p/4;
%%% derivative of interpolation function for r1
				Nr1(1)=-r2m/4;
				Nr1(2)= r2m/4;
				Nr1(3)= r2p/4;
				Nr1(4)=-r2p/4;
%%% derivative of interpolation function for r2
				Nr2(1)=-r1m/4;
				Nr2(2)=-r1p/4;
				Nr2(3)= r1p/4;
				Nr2(4)= r1m/4;
%%% G1,G2
				g1=zeros(3,1);
				g2=zeros(3,1);
				for k=1:4
					for l=1:3
						g1(l)=g1(l)+Nr1(k)*node(nL(k),l);
						g2(l)=g2(l)+Nr2(k)*node(nL(k),l);
					end
				end
				g3(1)=g1(2)*g2(3)-g1(3)*g2(2);
				g3(2)=g1(3)*g2(1)-g1(1)*g2(3);
				g3(3)=g1(1)*g2(2)-g1(2)*g2(1);
				size=norm(g3);
				g3=g3/size;
%%% Jacobi matrix
				J(1,1)=g1(1);
				J(1,2)=g1(2);
				J(1,3)=g1(3);
				J(2,1)=g2(1);
				J(2,2)=g2(2);
				J(2,3)=g3(3);
				J(3,1)=g3(1);
				J(3,2)=g3(2);
				J(3,3)=g3(3);
%%% weight at gauss point
				weight=wg(i)*wg(j)*det(J);
				for k=1:4
					vect(3*nL(k)-2)=vect(3*nL(k)-2)+val*weight*N(k)*g3(1);
					vect(3*nL(k)-1)=vect(3*nL(k)-1)+val*weight*N(k)*g3(2);
					vect(3*nL(k)  )=vect(3*nL(k)  )+val*weight*N(k)*g3(3);
				end
			end
		end
	end
%%% VOLUME LOAD
	if( ivol == 1 )
		loadL=zeros(8,1);
		for i=1:2
			r1=xg(i);
			for j=1:2
				r2=xg(j);
				for k=1:2
					r3=xg(k);
%%% interpolation function
					r1p=1+r1;
		  			r2p=1+r2;
		  			r3p=1+r3;
		  			r1m=1-r1;
		  			r2m=1-r2;
		  			r3m=1-r3;
		  			N(1)=r1m*r2m*r3m/8;
		  			N(2)=r1p*r2m*r3m/8;
		  			N(3)=r1p*r2p*r3m/8;
		  			N(4)=r1m*r2p*r3m/8;
		  			N(5)=r1m*r2m*r3p/8;
		  			N(6)=r1p*r2m*r3p/8;
		  			N(7)=r1p*r2p*r3p/8;
		  			N(8)=r1m*r2p*r3p/8;
%%% derivative of interpolation function  for r1
		  			Nr1(1)=-r2m*r3m/8;
		  			Nr1(2)= r2m*r3m/8;
		  			Nr1(3)= r2p*r3m/8;
		  			Nr1(4)=-r2p*r3m/8;
		  			Nr1(5)=-r2m*r3p/8;
		  			Nr1(6)= r2m*r3p/8;
		  			Nr1(7)= r2p*r3p/8;
		  			Nr1(8)=-r2p*r3p/8;
%%% derivative of interpolation function  for r2
		  			Nr2(1)=-r1m*r3m/8;
		  			Nr2(2)=-r1p*r3m/8;
		  			Nr2(3)= r1p*r3m/8;
		  			Nr2(4)= r1m*r3m/8;
		  			Nr2(5)=-r1m*r3p/8;
		  			Nr2(6)=-r1p*r3p/8;
		  			Nr2(7)= r1p*r3p/8;
		  			Nr2(8)= r1m*r3p/8;
%%% derivative of interpolation function  for r3
		  			Nr3(1)=-r1m*r2m/8;
		  			Nr3(2)=-r1p*r2m/8;
		  			Nr3(3)=-r1p*r2p/8;
		  			Nr3(4)=-r1m*r2p/8;
		  			Nr3(5)= r1m*r2m/8;
		  			Nr3(6)= r1p*r2m/8;
		  			Nr3(7)= r1p*r2p/8;
		  			Nr3(8)= r1m*r2p/8;
%%% Jacobi matrix 
					J=zeros(3,3);
					for l=1:8
						J(1,1)=J(1,1)+Nr1(l)*node(l,1);
						J(1,2)=J(1,2)+Nr1(l)*node(l,2);
						J(1,3)=J(1,3)+Nr1(l)*node(l,3);
						J(2,1)=J(2,1)+Nr2(l)*node(l,1);
						J(2,2)=J(2,2)+Nr2(l)*node(l,2);
						J(2,3)=J(2,3)+Nr2(l)*node(l,3);
						J(3,1)=J(3,1)+Nr3(l)*node(l,1);
						J(3,2)=J(3,2)+Nr3(l)*node(l,2);
						J(3,3)=J(3,3)+Nr3(l)*node(l,3);
					end
					wgt=wg(i)*wg(j)*wg(k)*det(J);
%%% loadL
					for l=1:8
						loadL(l)=loadL(l)+val*N(l)*wgt;
					end
				end
			end
		end
		switch( ltype )
			case (1)
			for i=1:8
				vect(3*i-2)=vect(3*i-2)+loadL(i);
			end
			case (2)
			for i=1:8
				vect(3*i-1)=vect(3*i-1)+loadL(i);
			end
			case (3)
			for i=1:8
				vect(3*i  )=vect(3*i  )+loadL(i);
			end
		end
	end
end
%%%
%%% VTK :: writeVTK
%%%
function writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,n_stress)
%%% VTK_CELL_TYPE 
	vtk_cell_type=12;
%%% NVARS
	NVARS=6; 
	vname=char("S11","S22","S33","S12","S23","S13");
%%% Header 
	fprintf(fid,"# vtk DataFile Version 2.0\n");
	fprintf(fid,"%s\n",program_name());
	fprintf(fid,"ASCII\n");
%%% Node 
	fprintf(fid,"DATASET UNSTRUCTURED_GRID\n");
	fprintf(fid,"POINTS %d float\n",numnp);
	for i=1:numnp
		xc(1)=0.0;xc(2)=0.0;xc(3)=0.0;
		for j=1:NDIM
			xc(j)=node(i,j);
		end
		fprintf(fid,"%e %e %e\n",xc(1),xc(2),xc(3));
	end
%%% Element
	fprintf(fid,"CELLS %d %d\n",nume,nume*(1+NN));
	for i=1:nume
		for j=1:NN
			nid(j)=element(i,j);
		end
		fprintf(fid,"%d ",NN);
		for j=1:NN
			fprintf(fid,"%d ",nid(j)-1);
		end
		fprintf(fid,"\n");
	end
%%% type
	fprintf(fid,"CELL_TYPES %d\n",nume);
	for i=1:nume
		fprintf(fid,"%d\n",vtk_cell_type);
	end
%%% Result 
	fprintf(fid,"POINT_DATA %d\n",numnp);
%%% Deformation 
	fprintf(fid,"VECTORS DISPLACEMENT float\n");
	for i=1:numnp
		if( NDIM == 3)
			fprintf(fid," %e %e %e\n",U(3*i-2),U(3*i-1),U(3*i));
		end
	end
%%% Nodal Stress 
	for i=1:NVARS
		fprintf(fid,"SCALARS %s float 1\n",vname(i,:));
		fprintf(fid,"LOOKUP_TABLE default\n");
		for j=1:numnp
			fprintf(fid,"%e\n",n_stress(j,i));
		end
	end
end
%%%
%%% Read FEM model data
%%%
function [numnp,node,nume,element,e,nu,nspc,spc,ncload,cload,vcload,ndload,dload,vdload]\
=readFEM(fid,NDIM,NN,NDOF)
%%%
%%% Node definitions:
%%%
numnp=fscanf(fid,"%d",'C');
nume=fscanf(fid,"%d",'C');
nspc=fscanf(fid,"%d",'C');
ncload=fscanf(fid,"%d",'C');
ndload=fscanf(fid,"%d",'C');
for i=1:numnp
	for j=1:NDIM
		node(i,j)=fscanf(fid,"%lf",'C');
	end
end
printf("///node :: numnp=%d\n",numnp);
for i=1:numnp
	printf("%d ",i);
	for j=1:NDIM
		printf("%e ",node(i,j));
	end
	printf("\n");
end
%%%
%%% Element definitions:
%%%
for i=1:nume
	dummy=fscanf(fid,"%d",'C');
	for j=1:NN
		element(i,j)=fscanf(fid,"%d",'C');
	end
end
printf("///element :: nume=%d\n",nume);
for i=1:nume
	printf("%d ",i);
	for j=1:NN
		printf("%d ",element(i,j) );
	end
	printf("\n");
end
%%%
%%% Material definitions:
%%%
e=fscanf(fid,"%lf",'C');
nu=fscanf(fid,"%lf",'C');
printf("///material :: E=%e NU=%e\n",e,nu);
%%%
%%% SPC definitions:
%%%
for i=1:nspc
	spc(i,1)=fscanf(fid,"%d",'C');
	spc(i,2)=fscanf(fid,"%d",'C');
end
printf("///spc :: nspc=%d\n",nspc);
for i=1:nspc
	printf("%d %d %d\n",i,spc(i,1),spc(i,2))
end
%%%
%%% load definitions:
%%%
cload(1,1)=0;
cload(1,2)=0;
vcload(1)=0.0;
for i=1:ncload
	cload(i,1)=fscanf(fid,"%d",'C');
	cload(i,2)=fscanf(fid,"%d",'C');
	vcload(i) =fscanf(fid,"%lf",'C');
end
printf("///cload :: ncload=%d\n",ncload);
for i=1:ncload
	printf("%d %d %d %e\n",i,cload(i,1),cload(i,2),vcload(i))
end
dload(1,1)=0;
dload(1,2)=0;
vdload(1)=0.0;
for i=1:ndload
	dload(i,1)=fscanf(fid,"%d",'C');
	dload(i,2)=fscanf(fid,"%d",'C');
	vdload(i)=fscanf(fid,"%lf",'C');
end
printf("///dload :: ndload=%d\n",ndload);
for i=1:ndload
	printf("%d %d %d %e\n",i,dload(i,1),dload(i,2),vdload(i))
end
end
%%%
%%%  Read FEM model data
%%%
finp=fopen('pwhC3D8.dat','r');
[numnp,node,nume,element,E,NU,nspc,spc,ncload,cload,vcload,ndload,dload,vdload]\
=readFEM(finp,NDIM,NN,NDOF);
%%%
%%% Global stiffness (NDOF*numnp) x (NDOF*numnp)
%%%
KG=zeros(NDOF*numnp,NDOF*numnp);
%%%
%%% Loop over elements
%%%
for ie=1:nume
%%% Local element coordinate
	for i=1:NN
		for j=1:NDIM
			nodeL(i,j)=node(element(ie,i),j);
		end
	end
%%% Local elememt stiffness element
	kL=k_hexa3d8(nodeL,E,NU);
%%% Add element 1 to Global stiffness 
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			for j=1:NN
				for jj=1:NDOF
					icol=NDOF*(element(ie,j)-1)+jj;
					KG(irow,icol)=KG(irow,icol)+kL(NDOF*(i-1)+ii,NDOF*(j-1)+jj);
				end
			end
		end
	end
end
%%%
%%% Load vector (condentrated load)
%%%
FC=zeros(NDOF*numnp,1);
for i=1:ncload
	npos=NDOF*(cload(i,1)-1)+cload(i,2);
	FC(npos)=FC(npos)+vcload(i);
end
%%%
%%% Load vector (distributed load)
%%%
FD=zeros(NDOF*numnp,1);
for i=1:ndload
	ie=dload(i,1);
	ltype=dload(i,2);
	val=vdload(i);
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
	end
	df=dload_hexa3d8(nodeL,ltype,val);
	for j=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,j)-1)+ii;
			FD(irow)=FD(irow)+df(NDOF*(j-1)+ii);
		end
	end
end
%%%
%%% TOTAL LOAD
%%%
	F=FC+FD;
%%%
%%% constrained conditions
%%%
count=1;
for i=1:nspc
	npos=NDOF*(spc(i,1)-1)+spc(i,2);
	row(count,:)=KG(npos,:);
	KG(npos,:)=0;
	KG(:,npos)=0;
	KG(npos,npos)=1;
	F(npos)=0.0;
	count=count+1;
end
%%%
%%% computation 
%%%
printf("### Computation started\n");
U=KG\F;
printf("### Computation finished\n");
printf("//Result displacement\n")
for i=1:numnp
	printf("%d ",i);
	for j=1:NDOF
		printf("%e ",U(NDOF*(i-1)+j));
	end
	printf("\n");
end
%%% post processing :: stress and strain
%%% for nodal stress/S11,S22,S33,S12,S23,S13
n_stress=zeros(numnp,6);
n_count=zeros(numnp);
r1v=[-1,-1,-1,-1, 1, 1, 1, 1];
r2v=[-1,-1, 1, 1,-1,-1, 1, 1];
r3v=[-1, 1, 1,-1,-1, 1, 1,-1];
rr=1/sqrt(3);
nvalue=zeros(8,6);
printf("Element stress/strain @Gauss points \n")
for ie=1:nume
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
		for k=1:NDOF
			dispL(j,k)=U(NDOF*(element(ie,j)-1)+k);
		end
		nid(j)=element(ie,j);
	end
	[s e]=rcv_hexa3d8(nodeL,E,NU,dispL);
	for ip=1:8
		printf("%d %d S11/S22/S33/S12/S23/S13: %e %e %e %e %e %e\n",\
        	ie,ip,s(ip,1),s(ip,2),s(ip,3),s(ip,4),s(ip,5),s(ip,6));
		printf("%d %d E11/E22/E33/E12/E23/E13: %e %e %e %e %e %e\n",\
        	ie,ip,e(ip,1),e(ip,2),e(ip,3),e(ip,4),e(ip,5),e(ip,6));
	end
	for np=1:8
		r1=r1v(np);
		r2=r2v(np);
		r3=r2v(np);
		r1/=rr;
		r2/=rr;
		r3/=rr;
		r1p=1+r1;
		r2p=1+r2;
		r3p=1+r3;
		r1m=1-r1;
		r2m=1-r2;
		r3m=1-r2;
		N(1)=r1m*r2m*r3m/8;
		N(2)=r1p*r2m*r3m/8;
		N(3)=r1p*r2p*r3m/8;
		N(4)=r1m*r2p*r3m/8;
		N(5)=r1m*r2m*r3p/8;
		N(6)=r1p*r2m*r3p/8;
		N(7)=r1p*r2p*r3p/8;
		N(8)=r1m*r2p*r3p/8;
		for k=1:6
			v(1)=s(1,k);
			v(2)=s(3,k);
			v(3)=s(4,k);
			v(4)=s(2,k);
			v(5)=s(2,k);
			v(6)=s(2,k);
			v(7)=s(2,k);
			v(8)=s(2,k);
			nvalue(np,k)=0.0;
			for l=1:8
				nvalue(np,k)=nvalue(np,k)+N(l)*v(l);
			end
		end
	end
%%% extrapolation to nodes
	for j=1:NN
		for k=1:4
			n_stress(nid(j),k)=n_stress(nid(j),k)+nvalue(j,k);
		end
		n_count(nid(j))=n_count(nid(j))+1;
	end
end
%%% average
for j=1:numnp
	if( n_count(j) > 0 )
		for k=1:4
			n_stress(j,k)=n_stress(j,k)/n_count(j);
		end
	end
end
printf("Nodal stress/S11 S22 S33 S12 S23 S13\n")
for i=1:numnp
	printf("%d %e %e %e %e %e %e\n",\
	i,n_stress(i,1),n_stress(i,2),n_stress(i,3),\
	n_stress(i,4),n_stress(i,5),n_stress(i,6));
end
%%% post processing :: reaction force
printf("Reaction force\n")
count=1;
for i=1:nspc
	f=-FD(NDOF*(spc(i,1)-1)+spc(i,2));
	for k=1:NDOF*numnp
		f=f+row(count,k)*U(k);
	end
	count=count+1;
	printf("%d %d %e\n",spc(i,1),spc(i,2),f);
end
%%%
%%% for visualization
%%%
	fid=fopen('vtk.vtk','w');
	writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,n_stress);
	fclose(fid);
%%%
%%% finished
%%%
